#ifndef CAPPLY_CPP
#define CAPPLY_CPP

// This is not the greatest class in the world, this is just an
// implementation.
// See COPYING for license information.

#include "CApply.h"

CApply::CApply(std::vector<struct_package> installs, std::vector<struct_package> removals, Glib::ustring mirror, Glib::ustring cache) :
_ApplyBut(Gtk::Stock::APPLY),
_DownloadBut("Only _Download", true),
_IOState(EAPPLY_IDLE),
_DLState(EAPPLY_NOTDOWNLOADING),
_CacheDir(cache),
_Downloader(NULL),
_Installer(NULL),
_Remover(NULL)
{	// Set up the UI to display all the packages and their pending operations.
	set_title("Pending Actions");
	set_border_width(6);
	set_default_size(600, 400);
	set_has_separator(false);

	// Create the treeview.
	_ColumnRecord.add(_ColName);
	_ColumnRecord.add(_ColStatus);
	_ColumnRecord.add(_ColSize);
	_ColumnRecord.add(_ColURL);
	_ColumnRecord.add(_ColCat);
	_ColumnRecord.add(_ColUpgrade);
	_RefModel = Gtk::ListStore::create(_ColumnRecord);
	_TreeView.set_model(_RefModel);
	_TreeView.append_column("Package", _ColName);
	_TreeView.append_column("Status", _ColStatus);
	_TreeView.append_column("Size", _ColSize);
	_TreeView.append_column("URL", _ColURL);
	_TreeView.append_column("Category", _ColCat);

	// Set up the vbox that holds the main widgets.
	get_vbox()->set_spacing(6);
	get_vbox()->pack_start(_Frame);
	_Frame.add(_Scroll);
	_Scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	_Scroll.add(_TreeView);
	get_vbox()->pack_start(_DLProgress, Gtk::PACK_SHRINK, Gtk::PACK_SHRINK);
	_DLProgress.set_text("Awaiting user input");
	get_vbox()->pack_start(_IOProgress, Gtk::PACK_SHRINK, Gtk::PACK_SHRINK);
	_IOProgress.set_text("Awaiting user input");

	// Set up the button box.
	add_button(Gtk::Stock::CLOSE, -4);
	get_action_area()->pack_start(_DownloadBut);
	_DownloadBut.set_tooltip_text("Only download packages to the local package cache, but do not install them");
	_DownloadBut.signal_clicked().connect(sigc::mem_fun(*this, &CApply::on_button_download));
	if ( !mirror.find("local://") ) _DownloadBut.set_sensitive(false);	// No need to download.
	get_action_area()->pack_start(_ApplyBut);
	_ApplyBut.signal_clicked().connect(sigc::mem_fun(*this, &CApply::on_button_apply));
	get_action_area()->set_layout(Gtk::BUTTONBOX_EDGE);

	// Set up the right-click menu.
	_TreeView.signal_button_press_event().connect_notify(sigc::mem_fun(*this, &CApply::on_buttonpress));
	_Menu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Remove from this list",
		sigc::mem_fun(*this, &CApply::on_menu_remove)));
	_Menu.items().push_back(Gtk::Menu_Helpers::MenuElem("Force _download",
		sigc::mem_fun(*this, &CApply::on_menu_download)));
	_Menu.accelerate(_TreeView);

	// Add data to the treeview.
	for ( std::vector<struct_package>::iterator _iter = installs.begin(); _iter != installs.end(); _iter++ ) {
		Gtk::TreeModel::Row	_row = *(_RefModel->append());
		_row[_ColName] = _iter->name;
		if ( !_iter->location.find("local://") ) _iter->location.erase(0, 8);
		if ( FileExists((_CacheDir + "/" + _iter->location + "/" + _iter->name + "-" + _iter->mirrorversion + _iter->type).c_str()) || !mirror.find("local://") )
			_row[_ColStatus] = "Cached";
		else	_row[_ColStatus] = "Download";
		_row[_ColSize] = formattedSize(_iter->packedsize);
		_row[_ColURL] = mirror + "/" + _iter->location + "/" + _iter->name + "-" + _iter->mirrorversion + _iter->type;
		_row[_ColCat] = _iter->location;
		if ( _iter->installedversion.length() ) _row[_ColUpgrade] = true;
		else	_row[_ColUpgrade] = false;
	}
	for ( std::vector<struct_package>::const_iterator _iter = removals.begin(); _iter != removals.end(); _iter++ ) {
		Gtk::TreeModel::Row	_row = *(_RefModel->append());
		_row[_ColName] = _iter->name;
		_row[_ColStatus] = "Remove";
		_row[_ColSize] = formattedSize(_iter->packedsize);
		_row[_ColURL] = "cache://" + _iter->name + "-" + _iter->installedversion;
	}

	show_all_children();
}

CApply::~CApply()
{	// Clean everything up, because this parade is OVER!
	delete _Downloader;
	delete _Installer;
	delete _Remover;
}

int CApply::Run()
{	// Check whether or not things have changed. If so, send the appropriate response.
	// If run() resulted in not the Close button, keep the form up.
	if ( run() == 1 ) return Run();
	_IOProgress.set_text("Searching for dotnew files...");
	_DLProgress.set_text("Searching for dotnew files...");
	while ( Gtk::Main::events_pending() ) Gtk::Main::iteration();
	if ( !_IOCurrentIter ) return 1;
	else if ( (*_IOCurrentIter)[_ColStatus] == "Installed" ||
		(*_IOCurrentIter)[_ColStatus] == "Removed" ||
		(*_IOCurrentIter)[_ColStatus] == "Installing..." ||
		(*_IOCurrentIter)[_ColStatus] == "Removing..." )
		return -4;	// Things have changed, so the display needs to be updated.
	else	return 1;
}

void CApply::on_button_download()
{	// Start a timer that only downloads packages, not installing them.
	_IOState = EAPPLY_IODONE;
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &CApply::on_timer), 200);
	_DownloadBut.set_sensitive(false);
	_ApplyBut.set_sensitive(false);
	_IOProgress.set_text("Downloading packages only, not installing...");
	_IOProgress.set_fraction(1.0f);
	WRITELOG("Downloading packages only, not installing...");
}

void CApply::on_button_apply()
{	// Start the timer that applies our pending actions.
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &CApply::on_timer), 200);
	_DownloadBut.set_sensitive(false);
	_ApplyBut.set_sensitive(false);
	WRITELOG("Applying changes...");
}

bool CApply::on_timer()
{	// Check how things are going, and start a new action if required.
	if ( _DLState == EAPPLY_NOTDOWNLOADING ) {
		// If we aren't downloading a file, check to see if we
		// should be and start one if that is the case.
		delete _Downloader;	// Take care of any possible leftovers.
		_Downloader = NULL;
		Gtk::TreeNodeChildren	_children = _RefModel->children();
		for ( Gtk::TreeNodeChildren::iterator _iter = _children.begin(); _iter != _children.end(); _iter++ ) {
			if ( (*_iter)[_ColStatus] == "Download" ) {
				_DLState = EAPPLY_DOWNLOADING;
				Glib::ustring	_url = (*_iter)[_ColURL];
				_Downloader = new CDownloader(_url, _CacheDir + "/" + (*_iter)[_ColCat] + _url.substr(_url.rfind("/")));
				_DLProgress.set_text("Downloading " + _url.substr(_url.rfind("/") + 1));
				_DLCurrentIter = _iter;
				(*_iter)[_ColStatus] = "Downloading...";
				return true;
			}
		}
		WRITELOG("Finished downloading/caching packages");
		_DLState = EAPPLY_DLDONE;
		_DLProgress.set_text("Finished downloading/caching packages");
		_DLProgress.set_fraction(1.0f);
	}
	if ( _IOState == EAPPLY_IDLE ) {
		// If we aren't installing or removing a files, then
		// start one if there are any pending ones left.
		delete _Installer;	// Take care of any leftovers.
		_Installer = NULL;
		delete _Remover;
		_Remover = NULL;
		Gtk::TreeNodeChildren	_children = _RefModel->children();
		for ( Gtk::TreeNodeChildren::iterator _iter = _children.begin(); _iter != _children.end(); _iter++ ) {
			if ( (*_iter)[_ColStatus] == "Cached" ) {
				_IOState = EAPPLY_INSTALLING;
				Glib::ustring	_pkg = (*_iter)[_ColURL];
				if ( !_pkg.find("local://") )
					_pkg = _pkg.substr(8);
				else	_pkg = _CacheDir + "/" + (*_iter)[_ColCat] + _pkg.substr(_pkg.rfind("/"));
				_Installer = new CPkgInstaller(_pkg, (*_iter)[_ColUpgrade]);
				if ( (*_iter)[_ColUpgrade] )
					_IOProgress.set_text("Updating " + _pkg.substr(_pkg.rfind("/") + 1));
				else	_IOProgress.set_text("Installing " + _pkg.substr(_pkg.rfind("/") + 1));
				_IOCurrentIter = _iter;
				(*_iter)[_ColStatus] = "Installing...";
				return true;
			}
		}
		for ( Gtk::TreeNodeChildren::iterator _iter = _children.begin(); _iter != _children.end(); _iter++ ) {
			if ( (*_iter)[_ColStatus] == "Remove" ) {
				_IOState = EAPPLY_REMOVING;
				Glib::ustring	_pkg = (*_iter)[_ColURL];
				_pkg = _pkg.substr(_pkg.rfind("/") + 1);
				_Remover = new CPkgRemover(_pkg);
				_IOProgress.set_text("Removing " + _pkg.substr(_pkg.rfind("/") + 1));
				_IOCurrentIter = _iter;
				(*_iter)[_ColStatus] = "Removing...";
				return true;
			}
		}
		if ( _DLState == EAPPLY_DLDONE ) _IOState = EAPPLY_IODONE;
		else	_IOProgress.set_text("Waiting for download to finish...");
	}
	if ( _DLState == EAPPLY_DOWNLOADING ) {	// Check the status of our download.
		float	_status = _Downloader->Iteration();
		if ( _status == -1.0f ) {
			_DLState = EAPPLY_NOTDOWNLOADING;
			(*_DLCurrentIter)[_ColStatus] = "Download Failed!";
		} else if ( _status == 2.0f ) {
			_DLProgress.set_fraction(1.0f);
			_DLState = EAPPLY_NOTDOWNLOADING;
			(*_DLCurrentIter)[_ColStatus] = "Cached";
		} else {
			_DLProgress.set_fraction(_status);
			Glib::ustring	_url = (*_DLCurrentIter)[_ColURL];
			_DLProgress.set_text("Downloading " + _url.substr(_url.rfind("/") + 1) + " -- " + _Downloader->GetSpeed() + "B/s");
		}
	}
	if ( _IOState == EAPPLY_INSTALLING ) {	// Check the status of installing/updating.
		float	_status = _Installer->Iteration();
		if ( _status == 2.0f ) {
			_IOProgress.set_fraction(1.0f);
			_IOState = EAPPLY_IDLE;
			(*_IOCurrentIter)[_ColStatus] = "Installed";
		} else if ( _status == -1.0f ) {
			_IOProgress.set_fraction(0.0f);
			_IOState = EAPPLY_IDLE;
			(*_IOCurrentIter)[_ColStatus] = "Failed";
		} else _IOProgress.pulse();
	} else if ( _IOState == EAPPLY_REMOVING ) {	// Check the status of removing.
		float	_status = _Remover->Iteration();
		if ( _status == 2.0f ) {
			_IOProgress.set_fraction(1.0f);
			_IOState = EAPPLY_IDLE;
			(*_IOCurrentIter)[_ColStatus] = "Removed";
		} else _IOProgress.pulse();
	}
	if ( _DLState == EAPPLY_DLDONE && _IOState == EAPPLY_IODONE ) {	// All done!
		WRITELOG("Finished applying changes");
		_IOProgress.set_text("Finished - Press Close to continue");
		notifyFinished();
		response(1);
		return false;
	}
	return true;
}

void CApply::on_buttonpress(GdkEventButton* eb)
{	// Check if 'eb' was a right-click, and if so show the menu.
	if ( !_TreeView.get_selection()->get_selected() ) return;
	if ( eb->type == GDK_BUTTON_PRESS && eb->button == 3 )
		_Menu.popup(0, gtk_get_current_event_time());
}

void CApply::on_menu_remove()
{	// Last-minute cancel performing an action on this package.
	Gtk::TreeModel::Row	_row = (*_TreeView.get_selection()->get_selected());
	_RefModel->erase(_row);
}

void CApply::on_menu_download()
{	// Force a package to be downloaded instead of getting it from cache.
	Gtk::TreeModel::Row	_row = (*_TreeView.get_selection()->get_selected());
	if ( _row[_ColStatus] == "Cached" )
		_row[_ColStatus] = "Download";
}

Glib::ustring CApply::formattedSize(guint32 size)
{	// Returns a nicely formatted string for package size. Takes size in KB.
	Glib::ustring	_ret = " KB";
	float		_size = (float)size;
	if ( _size >= 1024 && _size < 1048576 ) {
		_size /= 1024;
		_ret = " MB";
	} else if ( _size >= 1048576 ) {
		_size /= 1048576;
		_ret = " GB";
	}
	Glib::ustring	_tmpstr = Glib::Ascii::dtostr(_size);
	if ( (int)_tmpstr.find('.') >= 0 && (_tmpstr.length() - _tmpstr.find('.')) > 2 )
		_ret = _tmpstr.substr(0, _tmpstr.find(".") + 3) + _ret;	// Rounding time.
	else	_ret = _tmpstr + _ret;
	return _ret;
}

void CApply::notifyFinished()
{	// Displays a libnotify message when we are finished.
	unsetenv("DBUS_SESSION_BUS_ADDRESS");
	NotifyNotification*	_notnot;
	notify_init("Zendo");
	_notnot = notify_notification_new("Zendo finished", "Zendo has finished applying all changes", NULL, NULL);
	notify_notification_set_timeout(_notnot, 5000);
	notify_notification_show(_notnot, NULL);
	g_object_unref(G_OBJECT(_notnot));
}

#endif
