#ifndef CAPPLY_H
#define CAPPLY_H

#include <libnotify/notify.h>
#include <gtkmm.h>
#include <iostream>
#include <vector>
#include "CDownloader.h"
#include "CPkgInstaller.h"
#include "CPkgRemover.h"
#include "common_fun.h"
using namespace Common;

// Presents a dialog box of all the actions that will be performed
// on packages, and if the user accepts those actions it executes
// them and provides progress information.
// See COPYING for license information.

class CApply : public Gtk::Dialog
{
public:
	CApply(std::vector<struct_package> installs, std::vector<struct_package> removals, Glib::ustring mirror, Glib::ustring cache);
	~CApply();
	int	Run();
protected:
	void		on_button_download();
	void		on_button_apply();
	bool		on_timer();
	void		on_buttonpress(GdkEventButton* eb);
	void		on_menu_remove();
	void		on_menu_download();
	Glib::ustring	formattedSize(guint32 size);
	void		notifyFinished();

	Gtk::Frame				_Frame;
	Gtk::ScrolledWindow			_Scroll;
	Gtk::TreeView				_TreeView;
	Gtk::TreeModel::ColumnRecord		_ColumnRecord;
	Gtk::TreeModelColumn<bool>		_ColUpgrade;
	Gtk::TreeModelColumn<Glib::ustring>	_ColName;
	Gtk::TreeModelColumn<Glib::ustring>	_ColStatus;
	Gtk::TreeModelColumn<Glib::ustring>	_ColSize;
	Gtk::TreeModelColumn<Glib::ustring>	_ColURL;
	Gtk::TreeModelColumn<Glib::ustring>	_ColCat;
	Glib::RefPtr<Gtk::ListStore>		_RefModel;

	Gtk::ProgressBar	_IOProgress;
	Gtk::ProgressBar	_DLProgress;
	Gtk::Button		_ApplyBut;
	Gtk::Button		_DownloadBut;
	Gtk::Menu		_Menu;

	enum {	// State of the installing/removing process.
		EAPPLY_IDLE,
		EAPPLY_INSTALLING,
		EAPPLY_REMOVING,
		EAPPLY_IODONE
	} _IOState;
	enum {	// State of the downloading process.
		EAPPLY_DOWNLOADING,
		EAPPLY_NOTDOWNLOADING,
		EAPPLY_DLDONE
	} _DLState;

	CDownloader*			_Downloader;
	CPkgInstaller*			_Installer;
	CPkgRemover*			_Remover;
	Gtk::TreeNodeChildren::iterator	_IOCurrentIter;	// Which package is currently being installed/removed.
	Gtk::TreeNodeChildren::iterator	_DLCurrentIter;	// Which package is currently being downloaded.
	Glib::ustring			_CacheDir;
};

#endif
