#ifndef CDOWNLOADER_H
#define CDOWNLOADER_H

#include <iostream>
#include <fstream>
#include <stdlib.h>

// This class downloads a file using wget. To use it you
// need to call the constructor with a url and destination
// filename, then call Iteration() periodically to get the
// progress of the download, and to have this class check
// up on its status.
// See COPYING for license information.

class CDownloader
{
public:
	CDownloader(std::string url, std::string dest);
	~CDownloader();
	float		Iteration();
	std::string	GetSpeed();
protected:
	std::string	_Pid;
	std::string	_DestFile;
	bool		_Success;
	std::string	_CurSpeed;
};

CDownloader::CDownloader(std::string url, std::string dest) :
_Success(false),
_DestFile(dest)
{
	// Ensure the destination directory exists before we start.
	std::string	_cmd = "mkdir -p " + dest.substr(0, dest.rfind("/"));
	system(_cmd.c_str());

	// Setup our wget command, and execute it.
	_cmd = "wget " + url + " -o " + TMPFILE("download.log") + " -O " + dest + " &";
	system(_cmd.c_str());
	WRITELOG("Downloading: " + url);

	// Get the PID of the wget instance.
	_cmd = "ps aux | grep \"" + _cmd.substr(0, _cmd.length() - 2) + "\" > " + TMPFILE("wgetpid");
	system(_cmd.c_str());
	std::ifstream	_ifps;
	std::string	_inputstr;
	_ifps.open(TMPFILE("wgetpid").c_str(), std::ios_base::in);
	getline(_ifps, _inputstr);
	for ( int i = _inputstr.find(" "); i < _inputstr.length(); i++ ) {
		if ( _inputstr.substr(i, 1) != " " ) {
			_Pid = _inputstr.substr(i, (int)_inputstr.find(" ", i) - i);
			break;
		}
	}
	_ifps.close();
	_cmd = "rm -f " + TMPFILE("wgetpid");
	system(_cmd.c_str());
}

CDownloader::~CDownloader()
{	// If things didn't go too well, clean it up.
	if ( _Success ) return;
	std::string	_cmd = "kill " + _Pid + " 2> /dev/null";
	system(_cmd.c_str());
	_cmd = "rm -f " + _DestFile;
	system(_cmd.c_str());
}

float CDownloader::Iteration()
{	// Return current progress and perform regular actions.
	std::ifstream	_iflog;
	std::string	_inputstr;
	float		_progress = 0.0f;
	_iflog.open(TMPFILE("download.log").c_str(), std::ios_base::in);
	while ( !_iflog.eof() ) {
		getline(_iflog, _inputstr);
		if ( (int)_inputstr.find(" failed: ") >= 0 || (int)_inputstr.find(" ERROR ") >= 0 ) {
			WRITELOG("wget error: " + _inputstr);
			_progress = -1.0f;	// BUSTED!!
		} else if ( _progress > 0.0f && (int)_inputstr.find(_DestFile) >= 0 ) {
			WRITELOG("Finished downloading");
			_progress = 2.0f;	// File is finished
			_Success = true;
		} else if ( (int)_inputstr.find("% ") > 0 ) {
			_progress = strtod(_inputstr.substr(_inputstr.rfind(" ", _inputstr.find("% "))).c_str(), NULL);
			_progress /= 100.0f;
			if ( (int)_inputstr.find("K ", _inputstr.find("% ")) > 0 )	// Get current download speed.
				_CurSpeed = _inputstr.substr(_inputstr.rfind(" ", _inputstr.rfind("K")) + 1);
			else if ( (int)_inputstr.find("M ", _inputstr.find("% ")) > 0 )
				_CurSpeed = _inputstr.substr(_inputstr.rfind(" ", _inputstr.rfind("M")) + 1);
		}
	}
	_iflog.close();
	return _progress;
}

std::string CDownloader::GetSpeed()
{	// Return the current download speed.
	if ( (int)_CurSpeed.find(' ') >= 0 ) _CurSpeed.erase(_CurSpeed.find(' '));
	if ( !_CurSpeed.length() ) _CurSpeed = "0K";
	return _CurSpeed;
}

#endif
