#ifndef CVERSIONCOMPARE
#define CVERSIONCOMPARE
#include <iostream>

// A simple class to compare two version numbers of type std::string.
// It returns true if the first version number is greater than the
// second version number.  This method is not entirely fool-proof, but
// it does a handy enough job.  If the version numbers are equal it
// will return false.
// See COPYING for license information.

class CVersionCompare
{
public:
	CVersionCompare();
	bool	Compare(std::string version1, std::string version2);
protected:
	void	mangle(std::string& str);
};

CVersionCompare::CVersionCompare()
{
}

bool CVersionCompare::Compare(std::string version1, std::string version2)
{
	if ( version1.empty() || version2.empty() )	return false;
	mangle(version1);
	mangle(version2);

	// Seperate the build number from the version number.
	std::string	_build1;
	std::string	_build2;
	if ( (int)version1.find('!') >= 0 ) {
		_build1 = '.' + version1.substr(version1.find('!') + 1);
		version1.erase(version1.find('!'));
		version1 += '.';
	}
	if ( (int)version2.find('!') >= 0 ) {
		_build2 = '.' + version2.substr(version2.find('!') + 1);
		version2.erase(version2.find('!'));
		version2 += '.';
	}

	// Compare the version numbers.
	while ( version1.length() > 1 || version2.length() > 1 ) {
		int	_node1, _node2;

		if ( version1.find('.') == (version1.length() - 1) )
			_node1 = 0;
		else	_node1 = atoi(version1.substr(1, version1.find('.', 1)).c_str());
		if ( version2.find('.') == (version2.length() - 1) )
			_node2 = 0;
		else	_node2 = atoi(version2.substr(1, version2.find('.', 1)).c_str());

		if ( _node1 > _node2 ) return true;
		if ( _node2 > _node1 ) return false;

		if ( version1.length() > 1 ) version1 = version1.substr(version1.find('.', 1));
		if ( version2.length() > 1 ) version2 = version2.substr(version2.find('.', 1));
	}

	// Compare the build numbers.
	while ( _build1.length() > 1 || _build2.length() > 1 ) {
		int	_node1, _node2;

		if ( _build1.find('.') == (_build1.length() - 1) )
			_node1 = 0;
		else	_node1 = atoi(_build1.substr(1, _build1.find('.', 1)).c_str());
		if ( _build2.find('.') == (_build2.length() - 1) )
			_node2 = 0;
		else	_node2 = atoi(_build2.substr(1, _build2.find('.', 1)).c_str());

		if ( _node1 > _node2 ) return true;
		if ( _node2 > _node1 ) return false;

		if ( _build1.length() > 1 ) _build1 = _build1.substr(_build1.find('.', 1));
		if ( _build2.length() > 1 ) _build2 = _build2.substr(_build2.find('.', 1));
	}
	return false;	// The version numbers were equal, and so were the build numbers.
}

void CVersionCompare::mangle(std::string& str)
{
	std::transform(str.begin(), str.end(), str.begin(), tolower);

	if ( str.find('-') != str.rfind('-') )	// Erase arch.
		str.erase(str.find('-'), str.rfind('-') - str.find('-'));

	if ( (int)str.rfind('-') >= 0 )	// Seperate the build number from package number
		str.replace(str.rfind('-'), 1, "!");

	while ( (int)str.find('_') >= 0 )
		str.replace(str.find('_'), 1, ".");

	while ( (int)str.find('+') >= 0 )
		str.replace(str.find('+'), 1, ".");

	while ( (int)str.find("cvs") >= 0 )
		str.replace(str.find("cvs"), 3, ".");

	while ( (int)str.find("svn") >= 0 )
		str.replace(str.find("svn"), 3, ".");

	if ( (int)str.find("pre") >= 0 )
		str.replace(str.find("pre"), 3, ".-1.");

	if ( (int)str.find("rc") >= 0 )
		str.replace(str.find("rc"), 2, ".-1.");

	if ( (int)str.find('r') >= 0 )
		str.replace(str.find('r'), 1, ".-1.");

	if ( (int)str.find("beta") >= 0 )
		str.replace(str.find("beta"), 4, ".-2.");

	if ( (int)str.find('b') >= 0 )
		str.replace(str.find('b'), 1, ".-2.");

	if ( (int)str.find("alpha") >= 0 )
		str.replace(str.find("alpha"), 5, ".-3.");

	if ( (int)str.find('a') >= 0 )
		str.replace(str.find('a'), 1, ".-3.");

	for ( int i = 0; i < str.length(); i++ ) {	// Replace letters with numbers
		if ( str.c_str()[i] >= 'a' && str.c_str()[i] <= 'z' ) {
			char	_tmp[3];
			sprintf(_tmp, "%d", str.c_str()[i]);
			str.replace(i, 1, "." + std::string(_tmp) + "." );
		}
	}

	str = '.' + str + '.';
	while ( (int)str.find("..") >= 0 )
		str.erase(str.find(".."), 1);
}

#endif
