#ifndef CZENDO_CPP
#define CZENDO_CPP
// Implementation of the main Zendo window class.
// See COPYING for license information.

#include "CZendo.h"

CZendo::CZendo() :
_MainTable(7, 3),
_FiltersImage(PIXMAP("current_filter.png")),
_CurFilter(EALL)
{
	set_title(ZENDOTITLE);
	add(_Mainvbox);

	// Create the "File" menu
	_MenuBar.items().push_back(Gtk::Menu_Helpers::MenuElem("_File", _FileMenu));
	_FileMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Options", sigc::mem_fun(*this, &CZendo::on_menu_file_options)));
	_FileMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Load dependency file", sigc::mem_fun(*this, &CZendo::on_menu_file_loaddeps)));
	_FileMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Manage package cache", sigc::mem_fun(*this, &CZendo::on_menu_file_managecache)));
	_FileMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("Capture/Restore _State", sigc::mem_fun(*this, &CZendo::on_menu_file_state)));
	_FileMenu.items().push_back(Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::QUIT, sigc::mem_fun(*this, &CZendo::on_menu_file_quit)));
	// Create the "Filters" menu
	_MenuBar.items().push_back(Gtk::Menu_Helpers::MenuElem("Filter_s", _FiltersMenu));
	RebuildFiltersMenu(EALL);
	// Create the "Mirror" menu
	_MenuBar.items().push_back(Gtk::Menu_Helpers::MenuElem("_Mirror", _MirrorMenu));
	_MirrorMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Select/Edit Mirror", sigc::mem_fun(*this, &CZendo::on_menu_mirror_select)));
	_MirrorMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Reload Mirror", sigc::mem_fun(*this, &CZendo::on_menu_mirror_reload)));
	_MirrorMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Open Local Mirror", sigc::mem_fun(*this, &CZendo::on_menu_mirror_local)));
	// Create the "Help" menu
	_MenuBar.items().push_back(Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::HELP, _HelpMenu));
	_HelpMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Icon legend", sigc::mem_fun(*this, &CZendo::on_menu_help_iconlegend)));
	_HelpMenu.items().push_back(Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::ABOUT, sigc::mem_fun(*this, &CZendo::on_menu_help_about)));

	_Mainvbox.pack_start(_MenuBar, Gtk::PACK_SHRINK);

	// Set up the main widget table container
	_Mainvbox.pack_end(_MainTable);
	_MainTable.attach(_ApplyBut, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	  _ApplyBut.signal_clicked().connect(sigc::mem_fun(*this, &CZendo::on_button_apply));
	  _ApplyBut.set_tooltip_text("Apply pending actions");
	_MainTable.attach(_MarkAllBut, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	  _MarkAllBut.signal_clicked().connect(sigc::mem_fun(*this, &CZendo::on_button_mark_all));
	  _MarkAllBut.set_tooltip_text("Mark all packages in the list\nfor installation or update");	  
	_MainTable.attach(_MarkNoneBut, 2, 3, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	  _MarkNoneBut.signal_clicked().connect(sigc::mem_fun(*this, &CZendo::on_button_mark_none));
	  _MarkNoneBut.set_tooltip_text("Unmark all selections in the list");
	_MainTable.attach(_SearchVSep, 3, 4, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	_MainTable.attach(_ResetBut, 4, 5, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	  _ResetBut.signal_clicked().connect(sigc::mem_fun(*this, &CZendo::on_button_reset));
	  _ResetBut.set_tooltip_text("Clear the search box and relist the\npackages in the current category");
	_MainTable.attach(_SearchEntry, 5, 6, 0, 1, Gtk::SHRINK | Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK, 6, 6);
	  _SearchEntry.signal_activate().connect(sigc::mem_fun(*this, &CZendo::on_button_search));
	_MainTable.attach(_SearchBut, 6, 7, 0, 1, Gtk::SHRINK, Gtk::SHRINK | Gtk::FILL, 6, 6);
	  _SearchBut.signal_clicked().connect(sigc::mem_fun(*this, &CZendo::on_button_search));
	  _SearchBut.set_tooltip_text("Search for the specified text in package\nnames and optionally in descriptions");
	_MainTable.attach(_SelectorFrame, 0, 7, 1, 2, Gtk::SHRINK | Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK | Gtk::FILL | Gtk::EXPAND, 0, 0);
	  _SelectorFrame.add(_SelectorPane);
	    _SelectorPane.add1(_PkgCatView);
	    _SelectorPane.add2(_PkgPane);
	      _PkgPane.add1(_PkgList);
	      _PkgPane.add2(_PkgDescText);
	_MainTable.attach(_EventBox, 0, 7, 2, 3, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 0, 0);
	  _EventBox.add(_StatusBar);

	// Dirty hacked-together code to change the statusbar's color.
	GdkColor	_cgtk_color;
	GtkWidget*	_cgtk_widget;
	gdk_color_parse("White", &_cgtk_color);
	_cgtk_widget = _StatusBar.gobj()->label;
	gtk_widget_modify_fg(GTK_WIDGET(_cgtk_widget), GTK_STATE_NORMAL, &_cgtk_color);
	gdk_color_parse("#008080", &_cgtk_color);
	_cgtk_widget = _StatusBar.gobj()->frame;
	gtk_widget_modify_bg(GTK_WIDGET(_cgtk_widget), GTK_STATE_NORMAL, &_cgtk_color);
	_EventBox.modify_bg(Gtk::STATE_NORMAL, Gdk::Color("#008080"));

	show_all_children();

	loadSettings();

	// Set signal handlers and download the package list (if required).
	_PkgList.GetTreeView()->get_selection()->signal_changed().connect(sigc::mem_fun(*this, &CZendo::on_treeview_pkglist));
	_PkgCatView.GetTreeView()->get_selection()->signal_changed().connect(sigc::mem_fun(*this, &CZendo::on_treeview_pkgcat));
	_PkgList.Packages.clear();
	if ( !FileExists(TMPFILE("PACKAGES.TXT").c_str()) )	// Initial run.
		WRITELOG("PACKAGES.TXT does not exist.  It will be downloaded if necessary.");
	if ( _OptionsDlg.StartReloadChk.get_state() || !FileExists(TMPFILE("PACKAGES.TXT").c_str()) ) {
		if ( !refreshMirror() )	// Download PACKAGES.TXT automatically on startup.
			CRefresher	_parse(&_PkgList.Packages, _PkgList.Blacklist, _MirrorDlg.GetCurrentMirror());
	} else	CRefresher	_parse(&_PkgList.Packages, _PkgList.Blacklist, _MirrorDlg.GetCurrentMirror());

	// Parse the package list and create categories.
	if ( _OptionsDlg.MirrorCatChk.get_active() )
		_PkgCatView.BuildMirrorCatList(_PkgList.Packages);
	else	_PkgCatView.BuildNormalList(_PkgList.Packages);
	_PkgList.UpdateDisplay("All", _CurFilter);
	_StatusBar.push(_MirrorDlg.GetCurrentMirror() + "   (" + _PkgList.GetInstalledCount() + " installed/" + _PkgList.GetMirrorCount() + " on mirror)");
}

void CZendo::UpdateButtons(bool usetext)
{	// Create the button labels with or without text on them.
	if ( usetext ) {
		_ApplyBut.add_pixlabel(PIXMAP("apply.png"), "Apply");
		_MarkAllBut.add_pixlabel(PIXMAP("check_all.png"), "Mark All");
		_MarkNoneBut.add_pixlabel(PIXMAP("check_none.png"), "Unmark All");
		_SearchBut.add_pixlabel(PIXMAP("search.png"), "Search");
	} else {
		_ApplyBut.add_pixlabel(PIXMAP("apply.png"), "");
		_MarkAllBut.add_pixlabel(PIXMAP("check_all.png"), "");
		_MarkNoneBut.add_pixlabel(PIXMAP("check_none.png"), "");
		_SearchBut.add_pixlabel(PIXMAP("search.png"), "");
	}
	_ResetBut.add_pixlabel(PIXMAP("reset.png"), "");
}

void CZendo::RebuildFiltersMenu(int filter)
{	// (Re)create the filters menu with a marker next to the active one.
	static int	_lastfilter = -1;
	if ( _lastfilter == filter ) return;
	_lastfilter = filter;
	_FiltersMenu.items().clear();
	if ( filter == 0 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("_All", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_all)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_All", sigc::mem_fun(*this, &CZendo::on_menu_filters_all)));	
	if ( filter == 1 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("_Not Installed", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_notinstalled)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Not Installed", sigc::mem_fun(*this, &CZendo::on_menu_filters_notinstalled)));
	if ( filter == 2 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("_Installed", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_installed)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Installed", sigc::mem_fun(*this, &CZendo::on_menu_filters_installed)));
	if ( filter == 3 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("_Upgrades", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_upgrades)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Upgrades", sigc::mem_fun(*this, &CZendo::on_menu_filters_upgrades)));
	if ( filter == 4 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("Upgrades and _Downgrades", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_bothgrades)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("Upgrades and _Downgrades", sigc::mem_fun(*this, &CZendo::on_menu_filters_bothgrades)));
	if ( filter == 5 )
		_FiltersMenu.items().push_back(Gtk::Menu_Helpers::ImageMenuElem("_Blacklisted", _FiltersImage,
			sigc::mem_fun(*this, &CZendo::on_menu_filters_blacklisted)));
	else	_FiltersMenu.items().push_back(Gtk::Menu_Helpers::MenuElem("_Blacklisted", sigc::mem_fun(*this, &CZendo::on_menu_filters_blacklisted)));
}

void CZendo::on_menu_file_options()
{	// User wants to edit the options.
	bool	_oldschool = _OptionsDlg.MirrorCatChk.get_state();
	_OptionsDlg.run();
	_OptionsDlg.hide();
	Glib::ustring	_dir = _OptionsDlg.CacheDirEntry.get_text();
	if ( _dir.rfind("/") == _dir.length() - 1 )
		_OptionsDlg.CacheDirEntry.set_text(_dir.substr(0, _dir.length() - 1));
	if ( _oldschool == _OptionsDlg.MirrorCatChk.get_state() ) return;
	if ( _oldschool ) _PkgCatView.BuildNormalList(_PkgList.Packages);
	else _PkgCatView.BuildMirrorCatList(_PkgList.Packages);
}

void CZendo::on_menu_file_loaddeps()
{	// Load dependencies from a file, and select those packages.
	CDepFileDialog	_dlg(_PkgList.Packages);
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
}

void CZendo::on_menu_file_state()
{	// Capture/restore the state of your abacus of doom.
	CStateDialog	_dlg(_PkgList.Packages);
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
}

void CZendo::on_menu_mirror_select()
{	// User decided to select or edit the mirror list
	Glib::ustring	_tmpmir = _MirrorDlg.GetCurrentMirror();
	if ( _MirrorDlg.run() != 1 ) {
		_MirrorDlg.SetCurrentMirror(_tmpmir);
		_MirrorDlg.hide();
	} else {
		_MirrorDlg.hide();
		if ( !refreshMirror() ) {
			_MirrorDlg.SetCurrentMirror(_tmpmir);
			refreshMirror();
		}
	}
}

void CZendo::on_menu_mirror_local()
{	// Open a local mirror.
	CLocalMirror	_dlg;
	Glib::ustring	_dir = _dlg.Choose();
	if ( _dir.length() ) {
		_MirrorDlg.SetCurrentMirror(_dir);
		refreshMirror();
	}
}

void CZendo::on_menu_filters_all()
{	// Show all the packages from the mirror.
	_CurFilter = EALL;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_menu_filters_notinstalled()
{	// Show the packages from the mirror that we don't have installed.
	_CurFilter = ENEW;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_menu_filters_installed()
{	// Show all the packages installed on this computer.
	_CurFilter = EINSTALLED;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_menu_filters_upgrades()
{	// Show the packages from the mirror that are newer versions from those we have installed.
	_CurFilter = EUPGRADES;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_menu_filters_bothgrades()
{	// Show the packages from the mirror that are older versions from those we have installed.
	_CurFilter = EBOTHGRADES;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_menu_filters_blacklisted()
{	// Show only blacklisted packages.
	_CurFilter = EBLACKLISTED;
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	RebuildFiltersMenu(_CurFilter);
}

void CZendo::on_treeview_pkglist()
{	// User clicked a package, so change the description.
	Glib::ustring	_text;
	Glib::ustring	_pkg = _PkgList.GetCurrentPackageName();
	if ( !_pkg.length() ) return;	// No package selected. t3h 3x1t!!1!
	Glib::ustring	_deps = "Dependencies: " + _PkgList.Packages[_pkg].deps;
	_deps.erase(_deps.length() - 1);	// Don't display trailing comma.
	for ( gint16 i = 70; i < _deps.length(); i += 70 ) {
		if ( (gint32)_deps.rfind(',', i) > 0 )
			_deps.replace((gint32)_deps.rfind(',', i), 1, ",\n");
	}
	// The description must be converted to utf8 so that symbols like (r) are displayed properly.
	for ( int i = 0; i < _PkgList.Packages[_pkg].description.size(); i++ )
		_text += Glib::locale_to_utf8(_PkgList.Packages[_pkg].description[i]) + "\n";
	_text += "\nCompressed Size: " + Glib::Ascii::dtostr(_PkgList.Packages[_pkg].packedsize) + " KB\n";
	_text += "Uncompressed Size: " + Glib::Ascii::dtostr(_PkgList.Packages[_pkg].unpackedsize) + " KB\n";
	_text += "Original category: " + _PkgList.Packages[_pkg].location + "\n";
	_text += "Compression format: " + _PkgList.Packages[_pkg].type.substr(_PkgList.Packages[_pkg].type.find('.') + 1) + "\n";
	_text += "Conflicts: " + _PkgList.Packages[_pkg].cons + "\n";
	_text += _deps;
	_PkgDescText.SetText(_text);
}

void CZendo::on_treeview_pkgcat()
{	// Change to another package category, then update the description.
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	_PkgDescText.SetText("");
}

void CZendo::on_button_search()
{	// Search through packages, then update the description.
	Glib::ustring	_cat = _PkgCatView.GetCurrentCategory();
	pkgfilter_e	_filter = _CurFilter;
	if ( !_OptionsDlg.SearchCategoryChk.get_active() ) _cat = "All";
	if ( !_OptionsDlg.SearchFilterChk.get_active() ) _filter = EALL;
	_PkgList.UpdateDisplay(_cat, _filter, _SearchEntry.get_text(), _OptionsDlg.SearchDescChk.get_active());
	_PkgDescText.SetText("");
}

void CZendo::on_button_reset()
{	// Clear the search box, relist the packages.
	_SearchEntry.set_text("");
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
}

void CZendo::on_button_apply()
{	// Apply changes made. Download packages, install them, remove, then refresh.
	std::vector<struct_package>	_installpkgs;
	std::vector<struct_package>	_removepkgs;
	for ( packagemap_t::iterator _iter = _PkgList.Packages.begin(); _iter != _PkgList.Packages.end(); _iter++ ) {
		if ( _iter->second.color == COLOR_INSTALL )
			_installpkgs.push_back(_iter->second);
		else if ( _iter->second.color == COLOR_REMOVE )
			_removepkgs.push_back(_iter->second);
	}
	if ( !_installpkgs.size() && !_removepkgs.size() ) return;
	if ( _OptionsDlg.DependenciesChk.get_state() ) CDependencies _tmp(_installpkgs, _PkgList.Packages);
	CConflictsDialog	_consdlg(_installpkgs, _removepkgs, _PkgList.Packages);
	if ( !_consdlg.Continue ) return;	// There are conflicts.
	CApply	_applydlg(_installpkgs, _removepkgs, _MirrorDlg.GetCurrentMirror(), _OptionsDlg.CacheDirEntry.get_text());
	if ( _applydlg.Run() == 1 ) return;	// No need to refresh, nothing was done.
	_applydlg.hide();
	CDotnewDialog	_dotnewdlg(_installpkgs);
	_PkgList.Packages.clear();
	CRefresher	_refresh(&_PkgList.Packages, _PkgList.Blacklist, _MirrorDlg.GetCurrentMirror());
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	_SearchEntry.set_text("");	// Clear the search entry.
	_StatusBar.push(_MirrorDlg.GetCurrentMirror() + "   (" + _PkgList.GetInstalledCount() + " installed/" + _PkgList.GetMirrorCount() + " on mirror)");
}

void CZendo::loadSettings()
{
	std::ifstream	_ifconf;
	std::string	_inputstr;

	std::cout << "Loading settings...\n";
	_MirrorDlg.Clear();
	_ifconf.open("/etc/zendo.conf", std::ios_base::in);

	while ( _ifconf.is_open() && !_ifconf.eof() ) {
		getline(_ifconf, _inputstr);
		if ( !_inputstr.find("WindowSize=") )
			set_default_size((int)Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)),
				(int)Glib::Ascii::strtod(_inputstr.substr(_inputstr.find(':') + 1)));
		else if ( !_inputstr.find("SeperatorX=") )
			_PkgPane.set_position((int)Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("SeperatorY=") )
			_SelectorPane.set_position((int)Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("InstallDependencies=") )
			_OptionsDlg.DependenciesChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("MirrorCategories=") ) {
			_OptionsDlg.MirrorCatChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		} else if ( !_inputstr.find("SearchOnlyCategory=") )
			_OptionsDlg.SearchCategoryChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("SearchOnlyFilter=") )
			_OptionsDlg.SearchFilterChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("SearchDescription=") )
			_OptionsDlg.SearchDescChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("ReloadOnStart=") )
			_OptionsDlg.StartReloadChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
		else if ( !_inputstr.find("BlackList=") ) {
			_PkgList.Blacklist = _inputstr.substr(_inputstr.find('=') + 1);
			_PkgList.Blacklist = _PkgList.Blacklist.lowercase();
		} else if ( !_inputstr.find("Mirror=") )
			_MirrorDlg.AddMirror(_inputstr.substr(_inputstr.find('=') + 1));
		else if ( !_inputstr.find("CurrentMirror=") )
			_MirrorDlg.SetCurrentMirror(_inputstr.substr(_inputstr.find('=') + 1));
		else if ( !_inputstr.find("CacheDir=") )
			_OptionsDlg.CacheDirEntry.set_text(_inputstr.substr(_inputstr.find('=') + 1));
		else if ( !_inputstr.find("VerboseButtons=") ) {
			_OptionsDlg.VerboseButtonsChk.set_active(Glib::Ascii::strtod(_inputstr.substr(_inputstr.find('=') + 1)));
			UpdateButtons(_OptionsDlg.VerboseButtonsChk.get_active());
		}
	}
	_ifconf.close();
}

void CZendo::saveSettings()
{
	std::ofstream			_ofconf;
	int				xSize, ySize;

	std::cout << "Saving settings...\n";
	_ofconf.open("/etc/zendo.conf", std::ios_base::out);

	_ofconf << "# Zendo configuration file\n";
	_ofconf << "# For additional mirrors, please see:\n";
	_ofconf << "# http://zendo.naquadah.org/index.php?title=List_of_repositories\n\n";

	get_size(xSize, ySize);
	_ofconf << "WindowSize=" << Glib::Ascii::dtostr(xSize) << ":" << Glib::Ascii::dtostr(ySize) << "\n";
	_ofconf << "SeperatorX=" << _PkgPane.get_position() << "\n";
	_ofconf << "SeperatorY=" << _SelectorPane.get_position() << "\n\n";
	_ofconf << "InstallDependencies=" << _OptionsDlg.DependenciesChk.get_active() << "\n";
	_ofconf << "MirrorCategories=" << _OptionsDlg.MirrorCatChk.get_active() << "\n";
	_ofconf << "SearchOnlyCategory=" << _OptionsDlg.SearchCategoryChk.get_active() << "\n";
	_ofconf << "SearchOnlyFilter=" << _OptionsDlg.SearchFilterChk.get_active() << "\n";
	_ofconf << "SearchDescription=" << _OptionsDlg.SearchDescChk.get_active() << "\n";
	_ofconf << "ReloadOnStart=" << _OptionsDlg.StartReloadChk.get_active() << "\n\n";
	_ofconf << "# The BlackList is seperated by commas. e.g ,pkg1,pkg2,pkg3,pkg42," << "\n";
	_ofconf << "BlackList=" << _PkgList.Blacklist << "\n\n";

	_MirrorDlg.SaveMirrors(_ofconf);

	_ofconf << "\nCacheDir=" << _OptionsDlg.CacheDirEntry.get_text() << std::endl;
	_ofconf << "VerboseButtons=" << _OptionsDlg.VerboseButtonsChk.get_active() << std::endl;

	_ofconf.close();
}

bool CZendo::refreshMirror()
{	// Download the mirror list and parse it, return false if the user cancels out.
	_PkgList.Packages.clear();
	CRefresher	_refresh(_MirrorDlg.GetCurrentMirror());
	if ( _refresh.run() != 1 ) return false;
	_refresh.hide();
	CRefresher	_parse(&_PkgList.Packages, _PkgList.Blacklist, _MirrorDlg.GetCurrentMirror());
	if ( _OptionsDlg.MirrorCatChk.get_active() )
		_PkgCatView.BuildMirrorCatList(_PkgList.Packages);
	else	_PkgCatView.BuildNormalList(_PkgList.Packages);
	_PkgList.UpdateDisplay(_PkgCatView.GetCurrentCategory(), _CurFilter);
	_StatusBar.push(_MirrorDlg.GetCurrentMirror() + "   (" + _PkgList.GetInstalledCount() + " installed/" + _PkgList.GetMirrorCount() + " on mirror)");
	return true;
}

#endif
