#ifndef CZENDO_H
#define CZENDO_H

// Header file for the main Zendo window class.  This class also
// contains some inline implementation for very simple functions, so
// that the real implementation file will be as neat and concise as
// possible.
// See COPYING for license information.

#include <vector>
#include <gtk/gtk.h>
#include <gtkmm.h>
#include "CMirrorDialog.h"
#include "COptionsDialog.h"
#include "CApply.cpp"
#include "CRefresher.h"
#include "CAboutDialog.h"
#include "CDotnewDialog.h"
#include "CManageCacheDialog.h"
#include "CDepFileDialog.h"
#include "CConflictsDialog.h"
#include "CStateDialog.h"
#include "CDependencies.h"
#include "CLocalMirror.h"
#include "CGtkTextView.h"
#include "CGtkPkgCatView.h"
#include "CGtkPkgList.h"
#include "CIconDialog.h"
#include "common_fun.h"
using namespace Common;

// A '-' denotes something which has been completed.

//TODO: Mess around with orphan packages.  For real this time.
//TODO: Add ZUT functionality. Maybe.
//TODO: Add a built-in help system.
// - TODO: Add support for tlz, tbz, txz.

class CZendo : public Gtk::Window
{
public:
	CZendo();
	~CZendo() { saveSettings(); }
protected:
	void	on_menu_file_options();
	void	on_menu_file_loaddeps();
	void	on_menu_file_managecache() { CManageCacheDialog	_dlg(_OptionsDlg.CacheDirEntry.get_text()); }
	void	on_menu_file_state();
	void	on_menu_file_quit() { hide(); }
	void	on_menu_mirror_select();
	void	on_menu_mirror_reload() { refreshMirror(); }
	void	on_menu_mirror_local();
	void	on_menu_filters_all();
	void	on_menu_filters_notinstalled();
	void	on_menu_filters_installed();
	void	on_menu_filters_upgrades();
	void	on_menu_filters_bothgrades();
	void	on_menu_filters_blacklisted();
	void	on_menu_help_about() { CAboutDialog	_dlg; }
	void	on_menu_help_iconlegend() { CIconLegend	_dlg; }
	void	on_treeview_pkglist();
	void	on_treeview_pkgcat();
	void	on_button_search();
	void	on_button_reset();
	void	on_button_apply();
	void	on_button_mark_all() { _PkgList.MarkAll(); }
	void	on_button_mark_none() { _PkgList.MarkNone(); }
	void	UpdateButtons(bool usetext);
	void	RebuildFiltersMenu(int filter);
	void	loadSettings();
	void	saveSettings();
	bool	refreshMirror();

	CMirrorDialog	_MirrorDlg;
	COptionsDialog	_OptionsDlg;
	pkgfilter_e	_CurFilter;

	// Stuff for the menubar at the top of the program
	Gtk::VBox	_Mainvbox;
	Gtk::MenuBar	  _MenuBar;
	Gtk::Menu	  _FileMenu;
	Gtk::Menu	  _FiltersMenu;
	Gtk::Image	    _FiltersImage;
	Gtk::Menu	  _MirrorMenu;
	Gtk::Menu	  _HelpMenu;

	Gtk::Table	_MainTable;	// Table to hold all widgets
	Gtk::Button	  _ApplyBut;
	Gtk::Button	  _MarkAllBut;
	Gtk::Button	  _MarkNoneBut;
	Gtk::VSeparator	  _SearchVSep;	// Visually seperate the search stuff
	Gtk::Entry	  _SearchEntry;	// Entry to type search terms in
	Gtk::Button	  _SearchBut;	// Search button
	Gtk::Button	  _ResetBut;
	Gtk::Frame	  _SelectorFrame;	// Holds the package selection widgets
	Gtk::HPaned	    _SelectorPane;	// Holds the package selection widgets
	CGtkPkgCatView	      _PkgCatView;	// List of package categories
	Gtk::VPaned	      _PkgPane;		// Holds the package selection list and package description
	CGtkPkgList		_PkgList;	// List of packages
	CGtkTextView		_PkgDescText;	// Package description
	Gtk::EventBox	  _EventBox;	// Allows us to color the background of _StatusBar
	Gtk::Statusbar	    _StatusBar;	// Displays the URL of the current mirror
};

#endif
